@@ -28,10 +28,10 @@ import sys
 import time
 
 import enum
+from oslo_utils import encodeutils
 from oslo_utils import importutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import six
 
 from oslo_concurrency._i18n import _
 
@@ -41,8 +41,27 @@ from oslo_concurrency._i18n import _
 # time module as the check because that's a monkey patched module we use
 # in combination with subprocess below, so they need to match.
 eventlet = importutils.try_import('eventlet')
-if eventlet and eventlet.patcher.is_monkey_patched(time):
-    from eventlet.green import subprocess
+eventlet_patched = eventlet and eventlet.patcher.is_monkey_patched(time)
+if eventlet_patched:
+    if os.name == 'nt':
+        # subprocess.Popen.communicate will spawn two threads consuming
+        # stdout/stderr when passing data through stdin. We need to make
+        # sure that *native* threads will be used as pipes are blocking
+        # on Windows.
+        # Recent eventlet versions actually do patch subprocess.
+        subprocess = eventlet.patcher.original('subprocess')
+        subprocess.threading = eventlet.patcher.original('threading')
+    else:
+        from eventlet.green import subprocess
+
+    from eventlet import tpool
+
+    # Monkey patch the original current_thread to use the up-to-date _active
+    # global variable. See https://bugs.launchpad.net/bugs/1863021 and
+    # https://github.com/eventlet/eventlet/issues/592
+    import __original_module_threading as orig_threading
+    import threading
+    orig_threading.current_thread.__globals__['_active'] = threading._active
 else:
     import subprocess
 
@@ -161,7 +180,7 @@ class ProcessLimits(object):
     }
 
     def __init__(self, **kw):
-        for limit in self._LIMITS.keys():
+        for limit in self._LIMITS:
             setattr(self, limit, kw.pop(limit, None))
 
         if kw:
@@ -171,7 +190,7 @@ class ProcessLimits(object):
     def prlimit_args(self):
         """Create a list of arguments for the prlimit command line."""
         args = []
-        for limit in self._LIMITS.keys():
+        for limit in self._LIMITS:
             val = getattr(self, limit)
             if val is not None:
                 args.append("%s=%s" % (self._LIMITS[limit], val))
@@ -188,7 +207,7 @@ def execute(*cmd, **kwargs):
     :param cwd:             Set the current working directory
     :type cwd:              string
     :param process_input:   Send to opened process.
-    :type process_input:    string
+    :type process_input:    string or bytes
     :param env_variables:   Environment variables and their values that
                             will be set for the process.
     :type env_variables:    dict
@@ -249,16 +268,28 @@ def execute(*cmd, **kwargs):
     :param prlimit:         Set resource limits on the child process. See
                             below for a detailed description.
     :type prlimit:          :class:`ProcessLimits`
+    :param python_exec:     The python executable to use for enforcing
+                            prlimits. If this is not set it will default to use
+                            sys.executable.
+    :type python_exec:      string
+    :param timeout:         Timeout (in seconds) to wait for the process
+                            termination. If timeout is reached,
+                            :class:`subprocess.TimeoutExpired` is raised.
+    :type timeout:          int
     :returns:               (stdout, stderr) from process execution
     :raises:                :class:`UnknownArgumentError` on
                             receiving unknown arguments
     :raises:                :class:`ProcessExecutionError`
     :raises:                :class:`OSError`
+    :raises:                :class:`subprocess.TimeoutExpired`
 
     The *prlimit* parameter can be used to set resource limits on the child
     process.  If this parameter is used, the child process will be spawned by a
     wrapper process which will set limits before spawning the command.
 
+    .. versionchanged:: 3.17
+       *process_input* can now be either bytes or string on python3.
+
     .. versionchanged:: 3.4
        Added *prlimit* optional parameter.
 
@@ -266,7 +297,7 @@ def execute(*cmd, **kwargs):
        Added *cwd* optional parameter.
 
     .. versionchanged:: 1.9
-       Added *binary* optional parameter. On Python 3, *stdout* and *stdout*
+       Added *binary* optional parameter. On Python 3, *stdout* and *stderr*
        are now returned as Unicode strings by default, or bytes if *binary* is
        true.
 
@@ -279,6 +310,8 @@ def execute(*cmd, **kwargs):
 
     cwd = kwargs.pop('cwd', None)
     process_input = kwargs.pop('process_input', None)
+    if process_input is not None:
+        process_input = encodeutils.to_utf8(process_input)
     env_variables = kwargs.pop('env_variables', None)
     check_exit_code = kwargs.pop('check_exit_code', [0])
     ignore_exit_code = False
@@ -296,6 +329,8 @@ def execute(*cmd, **kwargs):
     on_completion = kwargs.pop('on_completion', None)
     preexec_fn = kwargs.pop('preexec_fn', None)
     prlimit = kwargs.pop('prlimit', None)
+    python_exec = kwargs.pop('python_exec', sys.executable)
+    timeout = kwargs.pop('timeout', None)
 
     if isinstance(check_exit_code, bool):
         ignore_exit_code = not check_exit_code
@@ -306,7 +341,7 @@ def execute(*cmd, **kwargs):
     if kwargs:
         raise UnknownArgumentError(_('Got unknown keyword args: %r') % kwargs)
 
-    if isinstance(log_errors, six.integer_types):
+    if isinstance(log_errors, int):
         log_errors = LogErrors(log_errors)
     if not isinstance(log_errors, LogErrors):
         raise InvalidArgumentError(_('Got invalid arg log_errors: %r') %
@@ -327,11 +362,16 @@ def execute(*cmd, **kwargs):
     cmd = [str(c) for c in cmd]
 
     if prlimit:
-        args = [sys.executable, '-m', 'oslo_concurrency.prlimit']
-        args.extend(prlimit.prlimit_args())
-        args.append('--')
-        args.extend(cmd)
-        cmd = args
+        if os.name == 'nt':
+            LOG.log(loglevel,
+                    _('Process resource limits are ignored as '
+                      'this feature is not supported on Windows.'))
+        else:
+            args = [python_exec, '-m', 'oslo_concurrency.prlimit']
+            args.extend(prlimit.prlimit_args())
+            args.append('--')
+            args.extend(cmd)
+            cmd = args
 
     sanitized_cmd = strutils.mask_password(' '.join(cmd))
 
@@ -358,7 +398,7 @@ def execute(*cmd, **kwargs):
                                    stderr=_PIPE,
                                    close_fds=close_fds,
                                    preexec_fn=on_preexec_fn,
-                                   shell=shell,
+                                   shell=shell,  # nosec:B604
                                    cwd=cwd,
                                    env=env_variables)
 
@@ -366,28 +406,40 @@ def execute(*cmd, **kwargs):
                 on_execute(obj)
 
             try:
-                result = obj.communicate(process_input)
+                # eventlet.green.subprocess is not really greenthread friendly
+                # on Windows. In order to avoid blocking other greenthreads,
+                # we have to wrap this call using tpool.
+                if eventlet_patched and os.name == 'nt':
+                    result = tpool.execute(obj.communicate,
+                                           process_input,
+                                           timeout=timeout)
+                else:
+                    result = obj.communicate(process_input,
+                                             timeout=timeout)
 
                 obj.stdin.close()  # pylint: disable=E1101
                 _returncode = obj.returncode  # pylint: disable=E1101
                 LOG.log(loglevel, 'CMD "%s" returned: %s in %0.3fs',
                         sanitized_cmd, _returncode, watch.elapsed())
+            except subprocess.TimeoutExpired:
+                LOG.log(loglevel, 'CMD "%s" reached timeout in %0.3fs',
+                        sanitized_cmd, watch.elapsed())
+                raise
             finally:
                 if on_completion:
                     on_completion(obj)
 
             if not ignore_exit_code and _returncode not in check_exit_code:
                 (stdout, stderr) = result
-                if six.PY3:
-                    stdout = os.fsdecode(stdout)
-                    stderr = os.fsdecode(stderr)
+                stdout = os.fsdecode(stdout)
+                stderr = os.fsdecode(stderr)
                 sanitized_stdout = strutils.mask_password(stdout)
                 sanitized_stderr = strutils.mask_password(stderr)
                 raise ProcessExecutionError(exit_code=_returncode,
                                             stdout=sanitized_stdout,
                                             stderr=sanitized_stderr,
                                             cmd=sanitized_cmd)
-            if six.PY3 and not binary and result is not None:
+            if not binary and result is not None:
                 (stdout, stderr) = result
                 # Decode from the locale using using the surrogateescape error
                 # handler (decoding cannot fail)
@@ -444,8 +496,10 @@ def trycmd(*args, **kwargs):
     the command's stdout and stderr.  If 'err' is not empty then the
     command can be considered to have failed.
 
-    :discard_warnings   True | False. Defaults to False. If set to True,
-                        then for succeeding commands, stderr is cleared
+    :param discard_warnings:  True | False. Defaults to False. If set to True,
+                              then for succeeding commands, stderr is cleared
+    :type discard_warnings:   boolean
+    :returns:                 (out, err) from process execution
 
     """
     discard_warnings = kwargs.pop('discard_warnings', False)
@@ -454,7 +508,7 @@ def trycmd(*args, **kwargs):
         out, err = execute(*args, **kwargs)
         failed = False
     except ProcessExecutionError as exn:
-        out, err = '', six.text_type(exn)
+        out, err = '', str(exn)
         failed = True
 
     if not failed and discard_warnings and err:
@@ -466,9 +520,21 @@ def trycmd(*args, **kwargs):
 
 def ssh_execute(ssh, cmd, process_input=None,
                 addl_env=None, check_exit_code=True,
-                binary=False, timeout=None):
+                binary=False, timeout=None,
+                sanitize_stdout=True):
     """Run a command through SSH.
 
+    :param ssh:             An SSH Connection object.
+    :param cmd:             The command string to run.
+    :param check_exit_code: If an exception should be raised for non-zero
+                            exit.
+    :param timeout:         Max time in secs to wait for command execution.
+    :param sanitize_stdout: Defaults to True. If set to True, stdout is
+                            sanitized i.e. any sensitive information like
+                            password in command output will be masked.
+    :returns:               (stdout, stderr) from command execution through
+                            SSH.
+
     .. versionchanged:: 1.9
        Added *binary* optional parameter.
     """
@@ -494,38 +560,35 @@ def ssh_execute(ssh, cmd, process_input=None,
 
     exit_status = channel.recv_exit_status()
 
-    if six.PY3:
-        # Decode from the locale using using the surrogateescape error handler
-        # (decoding cannot fail). Decode even if binary is True because
-        # mask_password() requires Unicode on Python 3
-        stdout = os.fsdecode(stdout)
-        stderr = os.fsdecode(stderr)
-    stdout = strutils.mask_password(stdout)
+    # Decode from the locale using using the surrogateescape error handler
+    # (decoding cannot fail). Decode even if binary is True because
+    # mask_password() requires Unicode on Python 3
+    stdout = os.fsdecode(stdout)
+    stderr = os.fsdecode(stderr)
+
+    if sanitize_stdout:
+        stdout = strutils.mask_password(stdout)
+
     stderr = strutils.mask_password(stderr)
 
     # exit_status == -1 if no exit code was returned
     if exit_status != -1:
         LOG.debug('Result was %s' % exit_status)
         if check_exit_code and exit_status != 0:
+            # In case of errors in command run, due to poor implementation of
+            # command executable program, there might be chance that it leaks
+            # sensitive information like password to stdout. In such cases
+            # stdout needs to be sanitized even though sanitize_stdout=False.
+            stdout = strutils.mask_password(stdout)
             raise ProcessExecutionError(exit_code=exit_status,
                                         stdout=stdout,
                                         stderr=stderr,
                                         cmd=sanitized_cmd)
 
     if binary:
-        if six.PY2:
-            # On Python 2, stdout is a bytes string if mask_password() failed
-            # to decode it, or an Unicode string otherwise. Encode to the
-            # default encoding (ASCII) because mask_password() decodes from
-            # the same encoding.
-            if isinstance(stdout, unicode):
-                stdout = stdout.encode()
-            if isinstance(stderr, unicode):
-                stderr = stderr.encode()
-        else:
-            # fsencode() is the reverse operation of fsdecode()
-            stdout = os.fsencode(stdout)
-            stderr = os.fsencode(stderr)
+        # fsencode() is the reverse operation of fsdecode()
+        stdout = os.fsencode(stdout)
+        stderr = os.fsencode(stderr)
 
     return (stdout, stderr)
 
@@ -533,8 +596,8 @@ def ssh_execute(ssh, cmd, process_input=None,
 def get_worker_count():
     """Utility to get the default worker count.
 
-    @return: The number of CPUs if that can be determined, else a default
-             worker count of 1 is returned.
+    :returns: The number of CPUs if that can be determined, else a default
+              worker count of 1 is returned.
     """
     try:
         return multiprocessing.cpu_count()
