@@ -13,19 +13,19 @@
 #    under the License.
 
 import collections
+import errno
 import multiprocessing
 import os
-import shutil
 import signal
 import subprocess
 import sys
 import tempfile
 import threading
 import time
+from unittest import mock
 
 from oslo_config import cfg
 from oslotest import base as test_base
-import six
 
 from oslo_concurrency.fixture import lockutils as fixtures
 from oslo_concurrency import lockutils
@@ -147,67 +147,95 @@ class LockTestCase(test_base.BaseTestCase):
         self.assertEqual(saved_sem_num, len(lockutils._semaphores),
                          "Semaphore leak detected")
 
+    def test_lock_internal_fair(self):
+        """Check that we're actually fair."""
+
+        def f(_id):
+            with lockutils.lock('testlock', 'test-',
+                                external=False, fair=True):
+                lock_holder.append(_id)
+
+        lock_holder = []
+        threads = []
+        # While holding the fair lock, spawn a bunch of threads that all try
+        # to acquire the lock.  They will all block.  Then release the lock
+        # and see what happens.
+        with lockutils.lock('testlock', 'test-', external=False, fair=True):
+            for i in range(10):
+                thread = threading.Thread(target=f, args=(i,))
+                threads.append(thread)
+                thread.start()
+                # Allow some time for the new thread to get queued onto the
+                # list of pending writers before continuing.  This is gross
+                # but there's no way around it without using knowledge of
+                # fasteners internals.
+                time.sleep(0.5)
+        # Wait for all threads.
+        for thread in threads:
+            thread.join()
+
+        self.assertEqual(10, len(lock_holder))
+        # Check that the threads each got the lock in fair order.
+        for i in range(10):
+            self.assertEqual(i, lock_holder[i])
+
+    def test_fair_lock_with_semaphore(self):
+        def do_test():
+            s = lockutils.Semaphores()
+            with lockutils.lock('testlock', 'test-', semaphores=s, fair=True):
+                pass
+        self.assertRaises(NotImplementedError, do_test)
+
     def test_nested_synchronized_external_works(self):
         """We can nest external syncs."""
-        tempdir = tempfile.mkdtemp()
-        try:
-            self.config(lock_path=tempdir, group='oslo_concurrency')
-            sentinel = object()
+        self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
+        sentinel = object()
 
-            @lockutils.synchronized('testlock1', 'test-', external=True)
-            def outer_lock():
+        @lockutils.synchronized('testlock1', 'test-', external=True)
+        def outer_lock():
 
-                @lockutils.synchronized('testlock2', 'test-', external=True)
-                def inner_lock():
-                    return sentinel
-                return inner_lock()
+            @lockutils.synchronized('testlock2', 'test-', external=True)
+            def inner_lock():
+                return sentinel
+            return inner_lock()
 
-            self.assertEqual(sentinel, outer_lock())
-
-        finally:
-            if os.path.exists(tempdir):
-                shutil.rmtree(tempdir)
+        self.assertEqual(sentinel, outer_lock())
 
     def _do_test_lock_externally(self):
         """We can lock across multiple processes."""
-        handles_dir = tempfile.mkdtemp()
-        try:
-            children = []
-            for n in range(50):
-                queue = multiprocessing.Queue()
-                proc = multiprocessing.Process(
-                    target=lock_files,
-                    args=(handles_dir, queue))
-                proc.start()
-                children.append((proc, queue))
-            for child, queue in children:
-                child.join()
-                count = queue.get(block=False)
-                self.assertEqual(50, count)
-        finally:
-            if os.path.exists(handles_dir):
-                shutil.rmtree(handles_dir, ignore_errors=True)
+        children = []
+        for n in range(50):
+            queue = multiprocessing.Queue()
+            proc = multiprocessing.Process(
+                target=lock_files,
+                args=(tempfile.mkdtemp(), queue))
+            proc.start()
+            children.append((proc, queue))
+        for child, queue in children:
+            child.join()
+            count = queue.get(block=False)
+            self.assertEqual(50, count)
 
     def test_lock_externally(self):
-        lock_dir = tempfile.mkdtemp()
-        self.config(lock_path=lock_dir, group='oslo_concurrency')
+        self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
 
-        try:
-            self._do_test_lock_externally()
-        finally:
-            if os.path.exists(lock_dir):
-                shutil.rmtree(lock_dir, ignore_errors=True)
+        self._do_test_lock_externally()
 
     def test_lock_externally_lock_dir_not_exist(self):
         lock_dir = tempfile.mkdtemp()
         os.rmdir(lock_dir)
         self.config(lock_path=lock_dir, group='oslo_concurrency')
 
-        try:
-            self._do_test_lock_externally()
-        finally:
-            if os.path.exists(lock_dir):
-                shutil.rmtree(lock_dir, ignore_errors=True)
+        self._do_test_lock_externally()
+
+    def test_lock_with_prefix(self):
+        # TODO(efried): Embetter this test
+        self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
+        foo = lockutils.lock_with_prefix('mypfix-')
+
+        with foo('mylock', external=True):
+            # We can't check much
+            pass
 
     def test_synchronized_with_prefix(self):
         lock_name = 'mylock'
@@ -225,115 +253,101 @@ class LockTestCase(test_base.BaseTestCase):
         self.assertTrue(bar(lock_dir, lock_pfix, lock_name))
 
     def test_synchronized_without_prefix(self):
-        lock_dir = tempfile.mkdtemp()
-        self.config(lock_path=lock_dir, group='oslo_concurrency')
+        self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
 
         @lockutils.synchronized('lock', external=True)
         def test_without_prefix():
             # We can't check much
             pass
 
-        try:
-            test_without_prefix()
-        finally:
-            if os.path.exists(lock_dir):
-                shutil.rmtree(lock_dir, ignore_errors=True)
+        test_without_prefix()
 
     def test_synchronized_prefix_without_hypen(self):
-        lock_dir = tempfile.mkdtemp()
-        self.config(lock_path=lock_dir, group='oslo_concurrency')
+        self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
 
         @lockutils.synchronized('lock', 'hypen', True)
         def test_without_hypen():
             # We can't check much
             pass
 
-        try:
-            test_without_hypen()
-        finally:
-            if os.path.exists(lock_dir):
-                shutil.rmtree(lock_dir, ignore_errors=True)
+        test_without_hypen()
 
     def test_contextlock(self):
-        lock_dir = tempfile.mkdtemp()
-        self.config(lock_path=lock_dir, group='oslo_concurrency')
+        self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
 
-        try:
-            # Note(flaper87): Lock is not external, which means
-            # a semaphore will be yielded
-            with lockutils.lock("test") as sem:
-                if six.PY2:
-                    self.assertTrue(isinstance(sem, threading._Semaphore))
-                else:
-                    self.assertTrue(isinstance(sem, threading.Semaphore))
+        # Note(flaper87): Lock is not external, which means
+        # a semaphore will be yielded
+        with lockutils.lock("test") as sem:
+            self.assertIsInstance(sem, threading.Semaphore)
 
-                # NOTE(flaper87): Lock is external so an InterProcessLock
-                # will be yielded.
-                with lockutils.lock("test2", external=True) as lock:
-                    self.assertTrue(lock.exists())
+            # NOTE(flaper87): Lock is external so an InterProcessLock
+            # will be yielded.
+            with lockutils.lock("test2", external=True) as lock:
+                self.assertTrue(lock.exists())
 
-                with lockutils.lock("test1",
-                                    external=True) as lock1:
-                    self.assertTrue(isinstance(lock1,
-                                               lockutils.InterProcessLock))
-        finally:
-            if os.path.exists(lock_dir):
-                shutil.rmtree(lock_dir, ignore_errors=True)
+            with lockutils.lock("test1", external=True) as lock1:
+                self.assertIsInstance(lock1, lockutils.InterProcessLock)
 
     def test_contextlock_unlocks(self):
-        lock_dir = tempfile.mkdtemp()
-        self.config(lock_path=lock_dir, group='oslo_concurrency')
+        self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
 
-        sem = None
+        with lockutils.lock("test") as sem:
+            self.assertIsInstance(sem, threading.Semaphore)
 
-        try:
-            with lockutils.lock("test") as sem:
-                if six.PY2:
-                    self.assertTrue(isinstance(sem, threading._Semaphore))
-                else:
-                    self.assertTrue(isinstance(sem, threading.Semaphore))
-
-                with lockutils.lock("test2", external=True) as lock:
-                    self.assertTrue(lock.exists())
-
-                # NOTE(flaper87): Lock should be free
-                with lockutils.lock("test2", external=True) as lock:
-                    self.assertTrue(lock.exists())
+            with lockutils.lock("test2", external=True) as lock:
+                self.assertTrue(lock.exists())
 
             # NOTE(flaper87): Lock should be free
-            # but semaphore should already exist.
-            with lockutils.lock("test") as sem2:
-                self.assertEqual(sem, sem2)
-        finally:
-            if os.path.exists(lock_dir):
-                shutil.rmtree(lock_dir, ignore_errors=True)
+            with lockutils.lock("test2", external=True) as lock:
+                self.assertTrue(lock.exists())
 
-    def _test_remove_lock_external_file(self, lock_dir, use_external=False):
-        lock_name = 'mylock'
-        lock_pfix = 'mypfix-remove-lock-test-'
+        # NOTE(flaper87): Lock should be free
+        # but semaphore should already exist.
+        with lockutils.lock("test") as sem2:
+            self.assertEqual(sem, sem2)
 
-        if use_external:
-            lock_path = lock_dir
-        else:
-            lock_path = None
+    @mock.patch('logging.Logger.info')
+    @mock.patch('os.remove')
+    @mock.patch('oslo_concurrency.lockutils._get_lock_path')
+    def test_remove_lock_external_file_exists(self, path_mock, remove_mock,
+                                              log_mock):
+        lockutils.remove_external_lock_file(mock.sentinel.name,
+                                            mock.sentinel.prefix,
+                                            mock.sentinel.lock_path)
 
-        lockutils.remove_external_lock_file(lock_name, lock_pfix, lock_path)
+        path_mock.assert_called_once_with(mock.sentinel.name,
+                                          mock.sentinel.prefix,
+                                          mock.sentinel.lock_path)
+        remove_mock.assert_called_once_with(path_mock.return_value)
+        log_mock.assert_not_called()
 
-        for ent in os.listdir(lock_dir):
-            self.assertRaises(OSError, ent.startswith, lock_pfix)
+    @mock.patch('logging.Logger.warning')
+    @mock.patch('os.remove', side_effect=OSError(errno.ENOENT, None))
+    @mock.patch('oslo_concurrency.lockutils._get_lock_path')
+    def test_remove_lock_external_file_doesnt_exists(self, path_mock,
+                                                     remove_mock, log_mock):
+        lockutils.remove_external_lock_file(mock.sentinel.name,
+                                            mock.sentinel.prefix,
+                                            mock.sentinel.lock_path)
+        path_mock.assert_called_once_with(mock.sentinel.name,
+                                          mock.sentinel.prefix,
+                                          mock.sentinel.lock_path)
+        remove_mock.assert_called_once_with(path_mock.return_value)
+        log_mock.assert_not_called()
 
-        if os.path.exists(lock_dir):
-            shutil.rmtree(lock_dir, ignore_errors=True)
-
-    def test_remove_lock_external_file(self):
-        lock_dir = tempfile.mkdtemp()
-        self.config(lock_path=lock_dir, group='oslo_concurrency')
-        self._test_remove_lock_external_file(lock_dir)
-
-    def test_remove_lock_external_file_lock_path(self):
-        lock_dir = tempfile.mkdtemp()
-        self._test_remove_lock_external_file(lock_dir,
-                                             use_external=True)
+    @mock.patch('logging.Logger.warning')
+    @mock.patch('os.remove', side_effect=OSError(errno.EPERM, None))
+    @mock.patch('oslo_concurrency.lockutils._get_lock_path')
+    def test_remove_lock_external_file_permission_error(
+            self, path_mock, remove_mock, log_mock):
+        lockutils.remove_external_lock_file(mock.sentinel.name,
+                                            mock.sentinel.prefix,
+                                            mock.sentinel.lock_path)
+        path_mock.assert_called_once_with(mock.sentinel.name,
+                                          mock.sentinel.prefix,
+                                          mock.sentinel.lock_path)
+        remove_mock.assert_called_once_with(path_mock.return_value)
+        log_mock.assert_called()
 
     def test_no_slash_in_b64(self):
         # base64(sha1(foobar)) has a slash in it
